package Abstract;

import java.net.*;
import java.io.*;
import java.util.*;

/**
* ServerThread is a thread which accept() connection from the other
* agents. Then it will generate ReceiverThread to receive and send message. 
* If the protocol and endWith character are not appropriate for your
* system, you can use setEndWitn(char) method to define end with charecter.
* ReceiverThread is just a thread for send/receive messages. No high level protocol is
* assumed on the ReceiverThread. 
* @see BaseLayer.BRecvThread, Abstract.ReceiverThread
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public abstract class ServerThread extends Connection {

  /**
  * ServerSocket object
  */
  protected ServerSocket _listenSocket;
  
  /**
  * Message Queue normally passed from AgentAction object
  * @see Abstract.MessageBuffer
  */
  protected MessageBuffer _queue;
  
	/**
	* Default constructor
	*/
	public ServerThread()  {
		_connections = null;
		_queue = null;
		// default endWith character is '\004'. Inherited from Connection class
		_endWith = '\004';
		// default duration time for the TCPRecvThread is not set. Inherited from 
		// Connection class
		_durationTime = -1;
		_security = null;
		_myaddress = null;
		_listenSocket = null;
	}
		 
  /**
  * Constructor using Address, ConnectionTable,MessageBuffer and Security objects.
  * @param address My address object which contains port number
  * @param connections connection table
  * @param mesg the message queue
  * @param security Security object which will be passed to the ReceiverThread
  * @exception ConnectionException, UnknownHostException, IOException
  */
  public ServerThread(Address address,
  											 ConnectionTable connections,
  										   MessageBuffer mesg,
  										   Security security)
	throws ConnectionException,UnknownHostException,IOException	   {
		_myaddress = address;
		_connections = connections;
		_queue = mesg;
		_security = security;
		_endWith = '\004';
		_durationTime = -1;	

		_listenSocket = new ServerSocket(_myaddress.getPort());
		if(_myaddress.getPort() == 0)  {
			_myaddress.setPort(_listenSocket.getLocalPort());
		}	
  }										   

  
  /**
  * Get message queue
  * @return MessageBuffer 
  */
  public MessageBuffer getMessageQueue()  {
  	return _queue;
  }
  
	/**
	* Set message queue
	* @param queue MessageBuffer
	*/
	public void setMessageQueue(MessageBuffer queue)  {
		_queue = queue;
	}
	
	/**
	* Set ServerSocket for the ServerThread 
	* @param s ServerSocket,opened for the ServerThread
	*/
	public void setServerSocket(ServerSocket s)  {
		_listenSocket = s;
	}
	
	/**
	* Get ServerSocket
	* @return ServerSocket, which is listening
	*/
	public ServerSocket getServerSocket()  {
		return _listenSocket;
	}
			
	/**
	* Listening and waiting for connection. If accepted, it will create ReceiverThread object. 
	* ConnectionTable will be automatically updated by Security object
	* The generated client receiver thread should be a subclass of ReceiverThread.
	* <B>Do not forget to set endWith character if the messages are not
	* ended with '\004'</B> 
	*/
  public void run()  {}
			
}
    					