package BaseLayer;

import java.util.Vector;

import Abstract.MessageBuffer;

/**
* Queue implementation.
* We do not specify the message type, which may be a String or
* special message type. 
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class BMessageBuffer implements MessageBuffer   {

	/**
	 * Internal storage for queue data.
	 */
	protected final Vector vec = new Vector();
	
	/**
	* Default constructor
	*/
	public BMessageBuffer()  {
	}


	public boolean isEmpty() { 
		synchronized ( vec ) { 
			return vec.isEmpty();
		}
	}

	public void removeAllMessages () { 
		synchronized ( vec ) { 
			vec.removeAllElements();
		}
	}

	/**
	* Retrieve head and remove it from the queue.
	* @return Object at head. If there's no element, it will return null
	*/
	public Object getMessage()   {
		Object obj = null;
		synchronized ( vec ) { 
			if( ! vec.isEmpty() )  {
				obj = vec.firstElement();
				vec.removeElementAt(0);
			}		
		}
		return obj;
	}


	public Object waitForMessage () { 
		while ( true ) { 
			synchronized( vec ) { 
				if ( ! vec.isEmpty() ) { 
					return getMessage();
				}
				try { vec.wait(); }
				catch ( final InterruptedException ie ) { ; }
			}
		}
	}

	public Object waitForMessage ( int time ) { 
		while ( true ) { 
			synchronized( vec ) { 
				if ( ! vec.isEmpty() ) { 
					return getMessage();
				}
				try { vec.wait( time ); }
				catch ( final InterruptedException ie ) { ; }
			}
		}
	}
	
	/**
	* Add message to the queue. Note that the formal parameter type is Object
	* @param o Object, added to the queue
	*/
	public void addMessage(Object o)  {
		synchronized ( vec ) {
			vec.addElement(o);
			vec.notify();  /* Notify a listener */
		}
	}


	/**
	* Sort via natural Comparable ordering.
	*/
	public void sortMessage(String type)  {
		synchronized ( vec ) { 
			java.util.Collections.sort( vec );
		}
	}
}	
				
