package BaseLayer;

import java.net.*;
import java.lang.*;

import Abstract.*;

/**
* Simple implementation of Security class.
* The register, unregister process is very simple. When connection is
* request from other agent, and accept() is successful, the server 
* thread will send 'Connected' message to the client. After the client
* received the 'Connected' message, the client will send its name
* to the server.
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 
public class BSecurity extends Security  {
	
	/**
	* Default constructor
	*/
  public BSecurity()  {
	}
	
	/**
	* Constructor with AddressTable
	* @param addresses AddressTable will be maintained by this security
	*/
	public BSecurity(AddressTable addresses)  {
		super(addresses);
	}
		
	/**
	* Constructor with AddressTable and PasswordFilePath
	* @param addresses AddressTable will be maintained by this security
	* @param passwordFilePath If you want password ckeck, you can specify its path
	*/	
	public BSecurity(AddressTable addresses,String passwordFilePath)  {
		super(addresses,passwordFilePath);
	}
	
	/**
	* Connection is requested from the client. Server will send
	* 'Connected' message and wait client id.
	* This will be override if new register protocol is added.
	* @param c TCPRecvThread will be started
	*/
	public void processServerLogin(ReceiverThread c)  {
		try  {
			c.writeMsg("Connected");
		}  catch (ConnectionException e)  {
			System.out.println("IO exception while writing. Connection closed");
			c.endConn();
			return;
		}
				
		String id = c.getOneMessage();	
		Address addr = new Address(id,"null",-1,"BRecvThread","(Client)");
		_addresses.addAddress(addr);
		c.getConnectionTable().addConnection(addr.getID(),c,true);
		System.out.println("Connection Request from " + id);				
  }
  
  /**
  * Implementation of processClientLogin method of Security class
  * Opposite to the processServerLogin. Connection should be added to the
  * connection table, by caller.
  * @param c BRecvThread, which will connect to the other server
  * @param serveraddress Address of the Server
  */
  public void processClientLogin(ReceiverThread c,Address serveraddress) 
  	throws ConnectionException   {
  		try  {
  			String con = c.getOneMessage();
  			if(con.equals("Connected"))  {
  				c.writeMsg(c.getAddress().getID());
  				c.getConnectionTable().addConnection(serveraddress.getID(),c,false);
  			}
  		}  catch (Exception e)  {
  			throw new ConnectionException("Security throws error " + e.toString());
  		}		
  }		

	/**
	* Send disconnect message to the server. The TCPRecvThread is initiated
	* by me.
	* @param c BRecvThread, which will stop.
	*/
  public void processClientLogout(ReceiverThread c)  {
  	try  {
  		c.writeMsg("Disconnect," + c.getAddress().getID());
  	}  catch (ConnectionException e)  {}		
  }			
  
  /**
  * No checking on the validity
  */
  public boolean isValidAgent(Object o) {
  	return true;
  }
}		
			
	
				