package BaseLayer;

import java.net.*;
import java.io.*;
import java.util.*;

import Abstract.*;

/**
* BServerThread is a thread which accept() connection from the other
* agents. Then it will generate BRecvThread to listen and send message
* For the protocol for connection, the Base layer assumes that when an agent
* want to connect to other agent, it will initiate connection then wait "Connected"
* message. If "Connected" message is received, it will imediately send its
* name to the other agent. The reverse connection is the same.
* Normally if the protocol and endWith character are not appropriate for your
* system, you can subclass this class and override run() method.
* BRecvThread is just a thread for send/receive messages.
* @see BaseLayer.Connection.BRecvThread
* @see Abstract.Connection.ReceiverThread
* <P>
*/
/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class BServerThread extends ServerThread {

	/**
	* Default constructor
	*/
	public BServerThread()  {
		super();
	}
		 
  /**
  * Constructor using Address object
  * @param address Address object which contains port number
  * @param connections connection table
  * @param mesg the message queue
  * @param security Security object to be used to check connection protocol
  */
  public BServerThread(Address address,
  											 ConnectionTable connections,
  										   MessageBuffer mesg,
  										   Security security)
	throws ConnectionException,UnknownHostException,IOException	   {
		super(address,connections,mesg,security);
	}	

	/**
	* Set duration time. Simply assign.
	* @param minutes Duration time will be set to all the generated Receiver Thread.
	*/
	public void setDurationTime(int minutes)  {
		_durationTime = minutes;
	}
		
	/**
	* Run method override
	* Waiting for accepting. If accepted, it will create BRecvThread object
	* and send "Connected" message to the client. Then the client will send 
	* its name. No other assumption on the protocol other than the above
	* description. You can override this method if you want more sophisticated
	* connection protocol.
	* ConnectionTable will be automatically updated.
	* The generated client receiver thread should be a subclass of BRecvThead.
	* It's default type is BRecvThread, but the extended type can be defined.
	* <B>Do not forget to set endWith character if the messages are not
	* ended with '\004'</B>
	*/
  public void run()  {
    
    try {
      while (true) {
				Socket client_socket = _listenSocket.accept();
				createReceiverThread(client_socket);
			}
    }  catch (IOException e) {
    }	
  } 

	/**
	* Create receiver thread using the socket, accepted by the 'run()' method.
	* @param socket Socket, accepted by the ServerSocket
	* @exception ConnectionException if occurs
	*/
  protected synchronized void createReceiverThread(Socket socket)  {
  	try  {
			BRecvThread c = new BRecvThread(socket,
									Thread.NORM_PRIORITY,_myaddress,_connections, _queue,_security);

			// do not forget to set end with character if it is not '\n'					
			c.setEndWith(_endWith);					
			if(_durationTime != -1)  {
				c.setDurationTime(_durationTime);
			}	
				
			c.start();
		} catch (ConnectionException c)  {
		}	
  } 	
  
  /**
  * Implements 'endConn()' method in Connection class. Close the socket and stop
  */
  public synchronized void endConn()  {
  	try   {
  		if(_listenSocket != null)  {
	  		_listenSocket.close();
  		}	
  	} catch (IOException e)  {}		
  		
  	stop();
  	try  {
 			join();
 		} catch (InterruptedException ee)  {}	
  		
  }

}
    					