package BaseLayer.Example;

import java.awt.*;
import java.io.*;
import java.applet.*;
import java.net.*;
import java.util.*;

import BaseLayer.*;
import Abstract.*;

/**
* BaseLayer example - Chatter. The html file should be modified to set the
* parameters.
*/
/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class ChatterApplet extends Applet  {

  TextField _userField;
  
	Choice _connection;
	TextArea _sendArea;
	TextArea _receiveArea;
	String _currentReceiver;
	ChatterAppletAction _agentAction;
	
	// My name
	String _userName;
	
	// chatter server id
	String _serverID;
	
  public ChatterApplet()   {
  	_userName = "";
  }

  public void init()  {
	
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();

		c.fill = GridBagConstraints.BOTH;
		c.weightx = 0.0;
		c.gridwidth = GridBagConstraints.REMAINDER;

		Panel textPanel = new Panel();
		textPanel.setLayout(new GridLayout(1,3));
		textPanel.add(new Label("AgentID"));
		_userField = new TextField("",10);
		textPanel.add(_userField);
			
		textPanel.add(new Button("Continue"));

		gridbag.setConstraints(textPanel,c);
		add(textPanel);

	
    Panel buttonPanel = new Panel();
    buttonPanel.setLayout(new GridLayout(1,4));
    
		c.gridwidth = GridBagConstraints.REMAINDER;
    
    _connection = new Choice();
    _connection.addItem("        ");
    _connection.addItem("All");
    buttonPanel.add(_connection);
    
    buttonPanel.add(new Button("Compose"));
    buttonPanel.add(new Button("Send"));
    buttonPanel.add(new Button("Quit"));
    
		gridbag.setConstraints(buttonPanel,c);
		add(buttonPanel);
    
		_receiveArea = new TextArea("Receiving Area\n",10,65);
		gridbag.setConstraints(_receiveArea,c);
		add(_receiveArea);
		
		_sendArea = new TextArea("Sending Area\n",10,65);
		gridbag.setConstraints(_sendArea,c);
		add(_sendArea);
  }

  public synchronized boolean handleEvent(Event e) {
    if(e.id == Event.WINDOW_ICONIFY)  {
        return true;
    }  else
    if(e.id == Event.WINDOW_DESTROY)  {
      return true;
    }  
    
    return super.handleEvent(e);
 }

  public boolean action(Event evt, Object arg) {
    if(evt.target instanceof Button)  {
    		if("Continue".equals((String)arg))  {
    			_userName = _userField.getText();
    			connectToServer();
    			notifyMessage("Try to connect to Server");
    		}
    		else	
        if("Compose".equals((String)arg))  {
        		_sendArea.setText("");
        }  
        else
        if("Send".equals((String)arg))  {
        		if(_currentReceiver != null)  {
            	String msg = _sendArea.getText();
            	try  {
            		_agentAction.sendMessage(_serverID,_userName + "\0" + _currentReceiver + "\0" + msg);
            	} catch (ConnectionException c)  {
            		System.out.println(c.toString());
            	}		
            }	
        } 
        else
        if("Quit".equals((String)arg))  {
        	  stop();
        }
     }  
     else  {
    	 _currentReceiver = _connection.getSelectedItem();
			notifyMessage("Receiver is set to " + _currentReceiver);    	 
     }	 	
     return true;
 }
 
 public void run()  {
 }
 
 public void processMessage(String command,String args[])  {
   try  {
		 if(command.equals("AddConnection"))  {
	 			String additional = args[0];
	 			if(additional.indexOf(',') != -1)  {
		 			StringTokenizer aToken = new StringTokenizer(additional,",");
	 				String id = "";
 					for(int i=0; i < aToken.countTokens(); i++)  {
		  			id = aToken.nextToken();
  					_connection.addItem(id);
  				}		
  				_connection.select(id);
  			}	
  			else  {
	  			_connection.addItem(args[0]);
  				_connection.select(args[0]);
  			}	
		}
		else
		if(command.equals("RemoveConnection"))  {
			String id	= args[0];
			//_connection.remove(id);
		}	 		
 	}  catch (Exception e)  {
	 	notifyMessage(e.toString());
 	}		
}	 

  protected void connectToServer()  {
    try  {
    		// chatter server name - "ChatterServer"
        _serverID = getParameter("SERVERNAME");
        String server = getParameter("SERVER");
        String port = getParameter("PORT");
        String type = getParameter("TYPE");
        String description = getParameter("DESCRIPTION");

				//_connection.remove("");
				_connection.addItem(_serverID);

  			if(_userName != "")   {     
					Address addr = new Address(_serverID,server,(Integer.valueOf(port)).intValue(),type,description);
					// server address is passed to connect as a client
					_agentAction = new ChatterAppletAction(addr,_userName);
					_agentAction.setApplet(this);
					_agentAction.setPriority(Thread.NORM_PRIORITY);
					
					_agentAction.start();
				}	
    } catch (Exception e)  {
        showStatus("Initialization Failed. Server not responding...");
        stop();
    }
  }

	public void notifyMessage(String msg)   {
		_receiveArea.appendText(_userName + ">" + msg + "\n");
	}
	

  public void stop()  {
  	try  {
 			_agentAction.sendMessage(_userName,_userName + "\0" + _serverID + "\0Quit"); 
 		} catch (ConnectionException c)  {
 			System.out.println(c.toString());
 		}		
  }

}