package BaseLayer.Example;

import BaseLayer.*;
import Abstract.*;

import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class ChatterServer extends BAgentAction  {
	
	public ChatterServer(String addressFilePath)   {
		super();
	
		try  {	
			// default processMessage is used 
			_connections = new ChatterConnectionTable(this);
			_queue = new BMessageBuffer();
			// read address file and store it to AddressTable
			BufferedReader input = new BufferedReader(new FileReader(new File(addressFilePath)));
			_addresses = new BAddressTable(input);
			input.close();
			
			// should be instantiated after addresstable is made
			_security = new BSecurity(_addresses);
			
			// Connect all the specified address
			// in our case, only one server
			Enumeration e = _addresses.elements();
			while(e.hasMoreElements())  {
				Address addr = (Address)e.nextElement();
				if(addr.getType().equals("Chatter"))  {
					// set my thread id
					String id = addr.getID();
					setName(id);
					createServerThread(id,Thread.NORM_PRIORITY);
				}
			}
		} catch (Exception e)  {
			System.out.println(e.toString());
		}		
	}
	
	// Connection change notification
	public void processMessage(String command,Object obj)  {
		String args[] = (String[])obj;
		if(command.equals("AddConnection"))  {
			if(_connections.size() != 1)  {
				String players = "";
				Enumeration e = _connections.keys();
				while(e.hasMoreElements())  {
					String id = (String)e.nextElement();
					if(!id.equals(args[0]))  {
						players = players + id + ",";
					}	
				}
				try  {
					sendMessage(args[0],"ChatterServer\0addConnection\0" + players.substring(0,players.length() -1 ));
				} catch (ConnectionException c)  {
					System.out.println(c.toString());
				}		
			}
			try  {	
				sendMessage(args[0],"ChatterServer\0introduction\0Welcome to JATLite chatter demo\n Select Receiver and send message\n");
				broadCast("ChatterServer\0addConnection\0" + args[0]);
			} catch (ConnectionException cc)  {
				System.out.println(cc.toString());
			}		
			
		}
		else
		if(command.equals("RemoveConnection"))  {
			try  {
				broadCast("ChatterServer\0removeConnection\0" + args[0]);
			} catch (ConnectionException ccc)  {
				System.out.println(ccc.toString());
			}		
		}
	}		
			
	protected boolean Act(Object o)  {
		String[] str = (String[])o;
		String sender = str[0];
		String receiver = str[1];
		String content = str[2];
		
		if(content.equals("Quit"))  {
			_connections.removeConnection(sender);	
		}
		else  {
			if(receiver.equals("All"))  {
				try  {
					broadCast(sender + "\0" + content);
				} catch (ConnectionException cc)  {
					System.out.println(cc.toString());
				}		
			}
			else	
			if(_connections.containsKey(receiver))  {
				try  {
					sendMessage(receiver,sender + "\0" + content);
				} catch (ConnectionException c)  {
					System.out.println(c.toString());	
				}	
			}
		}
		return true;
	}			
	
	// override. Just pass Message to the owner agent. 
	public void run()  {
		while(true)  {
				String newmsg = (String)_queue.waitForMessage();
				
				// Message format is sender,receiver,content
				StringTokenizer st = new StringTokenizer(newmsg,"\0");
		   	int nofToken = st.countTokens();
		   	if(nofToken == 3)  {
	     		String sender = st.nextToken();
	     		String receiver = st.nextToken();
	     		String content = st.nextToken();
	     		String[] str = new String[3];
	     		str[0] = sender;
	     		str[1] = receiver;
	     		str[2] = content;
		     	Act(str);
	     	}	
		}
	}	
	
	public static void main(String args[])  {
		if(args.length != 1)  {
			System.out.println("Please specify AddressFilePath");
			System.exit(1);
		}
		else  {
			ChatterServer chatter = new ChatterServer(args[0]);
			chatter.start();
			System.out.println("ChatterServer Started");
		}
	}					
}	
		
