package KQMLLayer.Example.SASync;

import java.lang.*;
import java.io.*;
import java.util.*;

import Abstract.*;
import BaseLayer.*;
import KQMLLayer.*;

import java.awt.*;

/**
* Example using SASyncWrapper. The received message will be echoed to
* the text area of the KQMLAgentFrameExample. The Act(object) method only
* redirect the KQMLmessage to the KQMLAgentFrameExample. No additional
* modification. 
* @version java(tm) 1.1b2
* @author Heecheol Jeon  01.05.96
*/
/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class SASyncExample extends KQMLAgentAction  {
	
  // Since this example use GUI, add _agentFrame 
  // Also, since GUI needs automatic updating for the connections,
  // connectionTable will need to have _agentFrame
  // Check SAConnectionTable() to see how it is extended
  SASyncFrameExample _agentFrame;
  public void setAgentFrame(SASyncFrameExample frame)  {
    _agentFrame = frame;
    ((SASyncConnectionTable)_connections).setAgentFrame(frame);
  }	
  
  /**
   * Default constructor
   */
  public SASyncExample()  {
    super();
  }
  
  /**
   * In the case of stand alone application
   * If your agent do not need AgentFrame, simply pass agentFrame as null
   * Read addresses from the AddressTable and try to make all the connections.
   * If ServerThread is failed, it will invoke processMessage method to
   * manage errors. 
   * @param id My name which represents myself
   * @param addressFilePath Address File Path
   * @param agentFrame KQMLAgentFrame, subclass of AgentFrame
   */
  public SASyncExample(String id,
		       String addressFilePath)  {
    
    super(id);
    
    try  {											
      
      _addresses = new BAddressTable();
      // dummy address, which may be overwritten when your agent name
      // is the same as the server agent name. It is save for you to
      // specify this dummy address when you construct an agent
      _addresses.addAddress(new Address(id,null,-1,"BRecvThread","Client"));
      
      initDataMembers();

      // KQMLAgentFrame
      SASyncFrameExample kqmlframe = new SASyncFrameExample(id,this);
      setAgentFrame(kqmlframe);
      kqmlframe.show();
      
      // Read AddressTable file and make AddressTable
      // When you have ServerThread, your ServerThread should appear
      // at the first line of the AddressTable.
      BufferedReader input = new BufferedReader(new FileReader(new File(addressFilePath)));
      String readline = input.readLine();
      while(readline != null)  {
				if(!readline.startsWith("#"))  {
	  			Address addr = new Address(readline);
	  			_addresses.addAddress(addr);
	  			String type = addr.getType();
	  			String name = addr.getID();
	  			if(type.equals("BServerThread"))   {
	    			ServerThread server = 
	      			createServerThread(name,Thread.NORM_PRIORITY);
	    			if(server == null)  {
	      			String args[] = new String[1];
	      			args[0] = addr.getID();
	      			processMessage("InitializeFailed",args);
	    			}	
	  			}	
	  			else
	    			if(type.equals("BRecvThread"))   {
	      			try  {
								createReceiverThread(name,Thread.NORM_PRIORITY);
	      			} catch (Exception e)  {
							// this error may happen if the server is not running
							// Ignore.
	      			}
	    			}
	    			else  {
	      			String args[] = new String[1];
	      			args[0] = type;
	      			processMessage("UnsupportedType",args);
	    			}	
				}
				readline = input.readLine();
      }							
      
      input.close();

    } catch (Exception ee)  {
      // critical error, can't start
      String args[] = new String[1];
      args[0] = ee.toString();
      processMessage("InitializeFailed",args);
    }		
  }
  
  /**
   *	Initialize data members
   */															
  protected void initDataMembers()   {
    // address table _addresses should not be null
    _security = new KQMLSecurity(_addresses);
    
    // ConnectionTable 
    _connections = new SASyncConnectionTable();
    
    // Message Queue
    _queue = new BMessageBuffer();
    
    //////////////////////////////////////////////////////////////
    // set all message should be ended with '\004' as default
    // If your application use different one set it.
    _endWith = '\004';
    //////////////////////////////////////////////////////////////
    
    //////////////////////////////////////////////////////////////
    // set receiver thread maximum duration time. Default is -1,
    // this means no setting(receiver thread running until your
    // application stops if no errors or disconnect message is 
    // received from other side connection)
    // If your application want to set, set maximum idle time,
    // unit is minute.
    _durationTime = -1;
    //////////////////////////////////////////////////////////////
  }
  
  /**
   * Extension of default processMessage method.
   * @param command command string
   * @param args[] argument list
   */
  public void processMessage(String command,Object obj)  {
    String[] args = (String[])obj;
    
    if(command.equals("UnsupportedType"))  {
      System.out.println(command + " " + args[0]);
    }
    else
      if(command.equals("InitializeFailed"))  {
	System.out.println(command + " " + args[0]);
	endAction();
      }
    // invoked from _agentFrame	
      else
	if(command.equals("Quit"))  {
	  endAction();
	}	
  }		
  
  
  /**
   * Override Act method. Get object from the MessageQueue and convert
   * it to KMQL object. Then perform appropriate action.
   * @param kqml KQMLmessage, received
   * @return true if action is successful, false if not successful.
   */	
  protected boolean Act(Object o)   {
    try  {
      KQMLmessage kqml = new KQMLmessage((String)o);
      
      String perf = kqml.getValue("performative");
      ////////////////////////////////
      // use performative,		
      // invoke appropriate method 
      ////////////////////////////////
      
      // For this example, agentFrame will echo the message to the text area
      _agentFrame.processMessage(kqml);
      /////////////////////////////////////////
      if(perf.equals("disconnected"))  {
				String args[] = new String[1];
				args[0] = kqml.getValue("sender");
				_connections.removeConnection(args[0]);
				_agentFrame.processMessage("RemoveConnection",args);
      }	
    } catch (ParseException e)  {
    }	
    return true;
  }
  
  /**
   * In general,you need not to change this. However, if you want to change priority
   * of the KQMLAgentAction or to hide the KMQLAgentFrame, you can change.
   * The first argument is your AgentName. If your addresses has ServerSocket,
   * the first argument should be the same as the server address name.
   * If your address has only clients, the first argument should not the
   * same as the name of the clients' address. The second argument is
   * the address file path.
   */
  public static void main(String args[])  {
    if(args.length == 2)  {
      try  {	
	// KQMLAgentAciton object. args[0]:my name args[1]:addressfilepath
	SASyncExample kqmlagent = new SASyncExample(args[0],args[1]);
	
	//////////////////////////////////////////////////////
	// If you want to change priority, change it
	kqmlagent.setPriority(Thread.NORM_PRIORITY);
	//////////////////////////////////////////////////////
	
	kqmlagent.start();
      } catch (Exception e)  {
	System.out.println(e.toString());
      }		
    }
    else  {
      System.out.println("AddressFilePath should be passed as arguments");
    }		
  }
  
  
}		
				