package KQMLLayer.Example.SASync;

import java.awt.*;
import java.awt.event.*;

import java.util.*;
import java.lang.*;

import BaseLayer.*;
import Abstract.*;
import KQMLLayer.*;

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 


public class SASyncFrameExample extends Frame implements ActionListener,ItemListener {

	AgentAction _agentAction;	
	
	Choice _connection;
	TextArea _sendArea;
	TextArea _receiveArea;
	String _currentReceiver;
	
	public SASyncFrameExample(String name,AgentAction awf)  {
		super(name);
		
		_agentAction = awf;
		
    Panel buttonPanel = new Panel();
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    
    buttonPanel.setLayout(gridbag);
    
    c.fill = GridBagConstraints.BOTH;
    c.weightx = 1.0;
    
    _connection = new Choice();
		_connection.addItemListener(this);
    // choice malfunction. Add two dummy choice
    _connection.add("");
    _connection.add("");
    
		ConnectionTable ctable = awf.getConnectionTable();
		Enumeration e = ctable.keys();
		while(e.hasMoreElements())  {
			String agentname = (String)e.nextElement();
			_connection.addItem(agentname);
		}	
		// set default as the first connection
		if(!ctable.isEmpty())  {
			_connection.select(0);
			_currentReceiver = _connection.getSelectedItem();
		}
		
		gridbag.setConstraints(_connection,c);	
    buttonPanel.add(_connection);

    c.weightx = 0.5;
    makeButton("Connect",gridbag,buttonPanel,c);
	  makeButton("Compose",gridbag,buttonPanel,c);
	  makeButton("Send",gridbag,buttonPanel,c);

    c.gridwidth = GridBagConstraints.REMAINDER;
		makeButton("Quit",gridbag,buttonPanel,c);
		add("North",buttonPanel);    
		
		_receiveArea = new TextArea("Receiving Area\n",10,50);
		_receiveArea.setEditable(false);
		add("Center",_receiveArea);
		
		_sendArea = new TextArea("Sending Area\n",10,50);
		add("South",_sendArea);

    setSize(400,400);
    setVisible(true);
  } 
  
  protected void makeButton(String name,GridBagLayout gridbag,Panel p,GridBagConstraints c) {
  	Button button = new Button(name);
  	button.addActionListener(this);
  	gridbag.setConstraints(button,c);
  	p.add(button);
  }	

	public void actionPerformed(ActionEvent evt)  {
		String arg = evt.getActionCommand();
		
    	  if("Connect".equals(arg))  {
    	  	  SASyncConnectionDlg connectionDlg = new SASyncConnectionDlg(this,_agentAction.getAddressTable(),_agentAction.getConnectionTable());
    	  	  connectionDlg.showDlg();
    	  }
    	  else	  
        if("Compose".equals(arg))  {
					String defaultMessage = "tell \r\n";
        	defaultMessage = defaultMessage + " :sender " + getTitle() + " \r\n";
        	defaultMessage = defaultMessage + " :receiver " + _currentReceiver + " \r\n";
        	defaultMessage = defaultMessage + " :language KQML \r\n";
        	defaultMessage = defaultMessage + " :content (       )";
        	_sendArea.setText(defaultMessage);
        }  
        else
        if("Send".equals(arg))  {
        	String sendstr = _sendArea.getText();
        	int strlen = sendstr.length();
        	String cstr = "(";
					cstr = cstr + filtering(sendstr) + ")";
					
					try  {
        		_agentAction.sendMessage(_currentReceiver,cstr);
        	} catch (ConnectionException c)  {
        		System.out.println(c.toString());
        	}		
	
/* just for test complete KQML syntax
char c = '\004';
String test = "(tell :receiver ";
test = test + _currentReceiver + " :content #10\"aab " + c + "ccdde :xxx yyy :zzz \"I am try to parse \\\"This thing \\\"\")";
_agentAction.sendMessage(_currentReceiver,test);
*/
    		}
        else
        if("Quit".equals(arg))  {
        		_agentAction.processMessage("Quit",null);
        	  dispose();
            System.exit(0);
        }
     
  }      

	public void itemStateChanged(ItemEvent evt)  {
   	_currentReceiver = _connection.getSelectedItem();
		notifyMessage("Receiver is set to " + _currentReceiver);    	 
	}
	
	
  protected String filtering(String sendstr) {
    String retstr ="";
    int strlen = sendstr.length();
    for(int i=0; i < strlen; i++)  {
			char c = sendstr.charAt(i);
			if((c != '\r') && (c != '\n')) {
	  	retstr = retstr + c;
	  	}	
	  }	
    return retstr;
  }
	  
	/**
	* Implement notifyError
	*/
	public void notifyError(String error)  {
		_receiveArea.append(error + "\n");
	}
	
	/**
	* Implement notifyMessage
	*/
	public void notifyMessage(String msg)   {
		_receiveArea.append(msg + "\n");
	}
			
	/**
	* Implement processMessage(String, String[])
	* Two command messages will be sent from ConnectionTable ->AgentAction
	* -> AgentFrame : AddConnection and RemoveConnection, which is performed
	* asynchronously, automatically by ConnectionTable.<BR>
	* The other messages will be sent from ConnectionDlg.
	*/
	public void processMessage(String command,String args[])  {
//		ActionWrapperWithFrame awwf = (ActionWrapperWithFrame)getAgentAction();
		// "Connect" to be invoked when this agent initialize a connection
		if(command.equals("Connect"))  {
System.out.println("Try to connect " + args[0]);		
			Address addr = new Address(args[0],args[1],(Integer.valueOf(args[2])).intValue(),args[3],args[4]);
			if(addr.getPort() == -1)   {
				notifyMessage("Receiver " + args[0] + " does not have ServerSocket and it is off-line. Connection aborted");
				return;
			}
				
			try  {	
				_agentAction.createReceiverThread(args[0],Thread.NORM_PRIORITY);
				notifyMessage("Receiver is set to " + args[0]);
System.out.println("Notify finish");				
//				addConnectionItem(args[0]);
			} catch (Exception e)  {
System.out.println(e.toString());			
				notifyError(e.toString());	
				
			}	
		}	
		else
		if(command.equals("Disconnect"))  {
			ConnectionTable ctable = _agentAction.getConnectionTable();
			ctable.removeConnection(args[0]);
		}		
		else	
		if(command.equals("RemoveConnection"))  {
				try  {
					_connection.remove(args[0]);
			} catch (Exception e)  {}	
		}		
		else
		if(command.equals("RemoveAddress"))  {
			AddressTable atable = _agentAction.getAddressTable();
			Address addr = atable.getAddress(args[0]);
			if(addr != null)  {
				atable.remove(addr.getID());
			}
		}
		else
		if(command.equals("AddAddress"))  {
			AddressTable atable = _agentAction.getAddressTable();
			Address addr = new Address(args[0],args[1],(Integer.valueOf(args[2])).intValue(),args[3],args[4]);
			// if same name address is already defined, remove
			Address tmp = atable.getAddress(args[0]);
			if(tmp != null)  {
				atable.remove(tmp.getID());
			}
			atable.addAddress(addr);
		}		
		else
		if(command.equals("AddConnection"))  {
			addConnectionItem(args[0]);
		}			
	}
					
	protected void addConnectionItem(String conid)  {
		_connection.add(conid);
		_connection.select(conid);
	}	

	/**
	* Implement processMessage(Object)
	*/
	public void processMessage(Object o)  {
		try  {
			KQMLmessage kqml = (KQMLmessage)o;
			String msg = kqml.getReadString();
			// There seems to be a bug in the 'append' method of TextArea.
			_receiveArea.setText("\r\n-----------------------------\r\n" + msg + "\r\n");
		} catch (Exception e)  {
			_receiveArea.append(e.toString());
		}
	}			
}			
																