package KQMLLayer;

import java.io.*;

  /** KQMLExpression is the representation of KQML expression
      KQMLExpression and its subclass provide a parser for a 
      KQML token stream as delivered by the KQMLStreamTokenizer
  */
/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <mailto:raquet@cdr.stanford.edu>Andreas Raquet</A>
*  @version Java(tm) 1.1
*/

public abstract class KQMLExpression {
  /** generate a KQML expression from a token stream
      @param parser is a scanner
   */
  public static KQMLExpression parse(KQMLStreamTokenizer parser) throws IOException, ParseException 
  {
    String token = parser.viewToken();
    if (token.equals("(")) return KQMLList.parse(parser);
    else if (token.equals("'")) return KQMLQuotation.parse(parser);
    else if (parser.ttype==StreamTokenizer.TT_WORD) return KQMLWord.parse(parser);
    else if ((char)parser.ttype=='"') return KQMLString.parse(parser);
    else if ((char)parser.ttype=='#') return KQMLString.parse(parser);
    else if ((char)parser.ttype==KQMLStreamTokenizer.TT_EOF) throw new ParseException("EOF found inside expression");
    else return KQMLWord.parse(parser);
  }
}