package KQMLLayer;

import java.util.Vector;
import java.util.StringTokenizer;
import java.util.Enumeration;

/**
* Utility to take off the left, right parenthesis and return the content as a Vector<BR>
* The quote expression '(a b) will be converted to (quote (a b)).
* The comma expression `(a b) will be converted to (listof (quote a) b).
* In the same line, the comma expression `a will be converted to (listof (quote a)).
* (a b (c d) ((e) (f g))) will be parsed to a Vector whose elements are<BR>
* a,b,(c d),((e) (f g))
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class takeOffList extends Vector   {

	/**
	* Default constructor
	*/
  public takeOffList()  {
		super();
  }

	/**
	* Constructor with string to be parsed
	* @param str String to be parsed
	* @exception ParseException if unpaired open,closed parenthesis
	*/
  public takeOffList(String str)  throws ParseException {

		super();  /* construct Vector */

		
		/* trim and convert qoute/comma expression to the standard form if exist */
		String nstr = convertStandardForm(str);
		
		try  {
			// should starts with open paren and ends with close paren
			if((nstr.startsWith("(")) && (nstr.endsWith(")")))  {
	   		nstr = nstr.substring(1,nstr.length()-1);
	   	}
	   	else  {
	   		throw new ParseException(str);
	   	}	
	   		
	   		nstr = nstr.trim();
	   		if(!nstr.equals("")) {    // empty

					char[] carray = nstr.toCharArray();
					
					int currentp = 0;
					String buf = "";
					int arraylen = carray.length;
				
					while(currentp < arraylen)  {
						
						char c = carray[currentp++];
						
						if(c == ' ')  {
							if(!buf.equals(""))	{
								addElement(buf);
								buf = "";
							}
						}
						else
						if(c == '(')  {
							int parno = 1;
							String tmp = "";
							tmp = tmp + c;
							while(parno != 0) {
								if(currentp > arraylen)  {
										throw new ParseException("Binary data out of bound::" + str);
								}	
							
								c = carray[currentp++];

								if(c == '(')  {
									tmp = tmp + c;
									parno++;
								}
								else
								if(c == ')')  {
									parno--;
									tmp = tmp + c;
								}
								else
								if(c == '#')  {
									String nobindata = "";
									nobindata = nobindata + c;

									String nofbin = "";
	
									while(true)  {
										c = carray[currentp++];
										if(currentp > arraylen)  {
											throw new ParseException("Binaryata out of bound::" + str);
										}	

										// if digit
										if((c >= '0')	&& (c <= '9'))  {
											nofbin = nofbin + c;
											nobindata = nobindata + c;
										}
										else
										if(c == '\"')  {
											if(!nofbin.equals("")) {
												int nofskip = (Integer.valueOf(nofbin)).intValue();
												
												if((currentp + nofskip) <= arraylen)  {
													String bindata = new String(carray,currentp,nofskip);
													tmp = tmp + nobindata + bindata;
													currentp += nofskip;
													break;
												}
												else  {
													throw new ParseException("Binary data out of bound::" + str);
												}
											}
											else  {
												throw new ParseException("Empty binary data::" + str);
											}
										}
										// not a digit binary data
										else  {
											tmp = tmp + nobindata + c;
											break;
										}
									}			
								}
								else
								// if double quote loop
								if(c == '\"')  {
									tmp = tmp + c;
						
									while(true)  {
										c = carray[currentp++];
										if(currentp > arraylen)  {
												throw new ParseException("Unmatched double quote::" + str);
										}	
							
							  		tmp = tmp + c;
							
										// add backslash character with backslash
										if(c == '\\')  {
											c = carray[currentp++];
											tmp = tmp + c;
										}
										else
										// string end	
										if(c == '\"')   {
//											tmp = tmp + c;
											break;
										}
									} // while	
								}	
								else  {
									tmp = tmp + c;
								}	
							}
							
							addElement(tmp);
							buf = "";
						}	
						else
						if(c == ')')  {
								throw new ParseException("Unmatched Parenthesis::" + str);
						}
						else
						if(c == '#')  {
								String nobindata = "";
								nobindata = nobindata + c;
	
								String nofbin = "";
								while(true)  {
									c = carray[currentp++];
									if(currentp > arraylen)  {
										throw new ParseException("Binary data out of bound::" + str);
									}	

									// if digit
									if((c >= '0')	&& (c <= '9'))  {
										nofbin = nofbin + c;
										nobindata = nobindata + c;
									}
									else
									if(c == '\"')  {
										if(!nofbin.equals("")) {
											int nofskip = (Integer.valueOf(nofbin)).intValue();
										
											if((currentp + nofskip) <= arraylen)  {
												String bindata = new String(carray,currentp,nofskip);
												addElement(bindata);
												currentp += nofskip;
												buf = "";
												break;
											}
											else  {
												throw new ParseException("Binary data out of bound::" + str);
											}
										}
										else  {
											throw new ParseException("Empty binary data::" + str);
										}
									}
									// not a digit binary data
									else  {
										buf = buf + nobindata + c;
										break;
									}
							}	
					}	
					// backslash
					else
					if(c == '\"')  {
						if(!buf.equals(""))  {
							throw new ParseException("Wrong double quote position::" + str);
						}	
						buf = buf + c;
						
						while(true)  {
							c = carray[currentp++];
							if(currentp > arraylen)  {
									throw new ParseException("Unmatched double quote::" + str);
							}	
							
							buf = buf + c;
							
							// add backslash character with backslash
							if(c == '\\')  {
								c = carray[currentp++];
								buf = buf + c;
							}
							else
							// string end	
							if(c == '\"')   {
								break;
							}
						} // while	
						
						addElement(buf);
						buf = "";
					}	
					else  {
						buf = buf + c;
					}
				
				}		// while
				if(!buf.equals(""))  {
					addElement(buf);
				}	
			}  // empty
	}	catch (Exception e)  {
			throw new ParseException(str);
		}
	}			
								
 
  protected String convertStandardForm(String str) 
  	throws ParseException   {

 		String sform = str.trim();
  	
  	try  {
  	
  		/* quote convert */
			int quoteindex = sform.indexOf('\'');
			while(quoteindex != -1)  {
				String head = sform.substring(0,quoteindex);
			
				if(sform.charAt(quoteindex + 1) == '(')  {
					int closeparnindex = findCloseParen(sform.substring(quoteindex + 2));
				
					// paran mismatch
					if(closeparnindex == -1)  {
						throw new ParseException(str);
					}
					

					String body = sform.substring(quoteindex + 2);
					sform = head + "(quote " + body;
				}
				else  {
					int spaceindex = findNextSpace(sform.substring(quoteindex + 1));
					quoteindex ++;
					String body = sform.substring(quoteindex,quoteindex + spaceindex);
					String tail = sform.substring(quoteindex + spaceindex);
					sform = head + "(quote " + body + ") " + tail;
						
				}
				quoteindex = sform.indexOf('\'',quoteindex);
			}	
			
			int bqindex = sform.indexOf('`');
			/* comma convert */
			while(bqindex != -1)  {
 				String head = sform.substring(0,bqindex);
			
				if(sform.charAt(bqindex + 1) == '(')  {
					int closeparnindex = findCloseParen(sform.substring(bqindex + 2));	
				
					if(closeparnindex == -1) {
						throw new ParseException(str);
					}
							  	
  				String body = sform.substring(bqindex + 2);
  				
  				// find relation
	  			int spaceindex = findNextSpace(body);
	  			String cmdbody = body.substring(0,spaceindex);
	  			String listbody = convertCommaToSpace(body.substring(spaceindex + 1));
  				sform = head + "(listof (quote " + cmdbody + ") " + listbody;	  		
  			}
  			else  {
					int spaceindex = findNextSpace(sform.substring(bqindex + 1));
					bqindex++;
					String body = sform.substring(bqindex,bqindex + spaceindex);
	  			String cmdbody = body.substring(0,spaceindex);
	  			String tail = sform.substring(bqindex + spaceindex);
					sform = head + "(listof (quote " + cmdbody + ")) " + tail;
				}  					
  			bqindex = sform.indexOf('`');
	 		}			
  	} catch (Exception e)  {
  		throw new ParseException(str);
  	}
  		
  	return sform;
  }		
  
  protected int findCloseParen(String str)  {

		char[] carray = str.toCharArray();
		int parseno = 1;
		for(int i=0; i<carray.length; i++)  {
			char c = carray[i];
			if(c == '(')  parseno++;
			else
			if(c == ')')  {
				parseno--;
				if(parseno == 0)  return i;
			}
		}
		
		return -1;
	}					
  	
  protected int findNextSpace(String str)  {
  	char[] carray = str.toCharArray();
  	for(int i=0; i< carray.length; i++)  {
  		char c = carray[i];
  		if((c == ' ') || (c == ')'))  {
  			return i;
  		}
  	}
  	
  	return carray.length;
  }
  				
  protected String convertCommaToSpace(String str)  {
  	int parseno = 0;
  	char[] carray = str.toCharArray();
  	
  	for(int i=0; i < carray.length; i++)  {
  		char c = carray[i];
  		if(c == ',')  {
  			if(parseno == 0)  {
  				carray[i] = ' ';
  			}
  		}
  		else
  		if(c == '(')  {
  			parseno++;
  		}
  		else
  		if(c == ')')  {
  			parseno--;
  		}
  	}
  	
  	return new String(carray);
  }					
  			
  public String toReadString()  {
  	String retstr = "";
  	Enumeration e = elements();
  	while(e.hasMoreElements())  {
  		retstr = retstr + (String)e.nextElement() + ";";
  	}
  	
  	return retstr;
  }		
  
  static public void main(String args[])  {
  	String qtest = "'(a 'b)";
  	String btest2 = "(\"aaa )()()( \\\" ccc\")";
  	String btest = "(tell :send xxx :xxx \"aaa \\\"ccc \\\"eee fff\\\" ddd\\\" bbb\" :recv yyy :content (#10\"a\"\"\004)))((c))";
  	String btest1 = "'(a #10\"aaa\004)))ccc :content #5\")))(()";
  	String exam1 = "(true (subst (name `(subst (name x) `x `(true ,x))) `x `(not (true (subst (name x) `x `(not (true ,x)))))))";
  	String ctest = "(tell :sender xxx :receiver yyy :content `(comma ,a,b,`(material b c) `m 'k `z))";
  	takeOffList v = null;
  	try  {
  		v = new takeOffList(btest2);
  	} catch (ParseException e)  {
  		System.out.println(e.toString());
  	}		
  	System.out.println(v.toReadString());
  }			
}