package ProtocolLayer;

import Abstract.*;
import BaseLayer.*;

import java.net.*;
import java.lang.*;


/**
  * FTP security managing module. JATLite beta only implements FTP client side,
  * All necessary steps to transfer data will be performed here - user name/
  * password check and directory existence. For any security violation such as
  * wrong user name, password or directory not exist will throw an exception.
  */

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class FTPSecurity extends Security   {
	
	/**
	  * Remote FTP site initial path
	  */
	String _remotePath;
	
	/**
	  * FTP server user login name(not agent name for router)
	  */
	String _userName;
	
	/**
	  * FTP server user login password(not agent password for router)
	  */
	String _password;
	
	/**
	  * Create directory for remote site if true
	  */
	boolean _createMode;  // create directory if not exist
	
	/**
	* Default constructor. 
	*/
  public FTPSecurity()  {
		super();
		_remotePath = "";
		_userName = "anonymous";
		_password = "";
	}
	
	/**
	* FTP security for checking username, password and remote path accessibility
	*/
	public FTPSecurity(String remotePath,String userName,String password,boolean createDir) {
		_remotePath = remotePath;
		_userName = userName;
		_password = password;
		_createMode = createDir;
	}	
	
	/**
	* SMTP is only for client side security check. No implementation. 
	* This will be override if new register protocol is added.
	* @param c ReceiverThread will be started
	*/
	public void processServerLogin(ReceiverThread c) {}; 
	
  /**
	* FTP connection protocol. If not satisfied or error occurs, ConnectionException
	* will be thrown.
  * @param c ReceiverThread, which will connect to the other server
  * @param serveraddress Other agent address where I want to connect
  */
  public void processClientLogin(ReceiverThread c,Address serveraddress) 
  	throws ConnectionException {
  	if(serveraddress == null)  {
  		System.out.println("Null FTP server address is passed");
  		throw new ConnectionException("Null FTP server address is passed");
  	}
  	
  	String retstr = c.getOneMessage();
System.out.println(retstr);  	
  	if(!retstr.startsWith("220"))  {
  		System.out.println("Can not connect to FTP server " + serveraddress.getHost());
  		((FTPThread)c).notifyError("Can not connect to FTP server " + serveraddress.getHost());
  		throw new ConnectionException("Can not connect to FTP server " + serveraddress.getHost());
		}
		  		
  	String opencmd = "user " + _userName;
  	c.writeMsg(opencmd);
System.out.println(opencmd);  	
  	retstr = c.getOneMessage();
System.out.println(retstr);  	
  	
  	retstr = retstr.substring(0,3);
  	
  	// 230 success, skip
  	if(retstr.equals("331"))  {
  		if(_password.equals(""))  {
  			_password = "JATLite@cdr";
  		}	
  		opencmd = "pass " + _password;
  		c.writeMsg(opencmd);
System.out.println(opencmd);  	
  		
	  	retstr = c.getOneMessage();		
System.out.println(retstr);  	
	  	
  		if(!retstr.startsWith("230"))  {
  			System.out.println("FTPError <" + serveraddress.getHost() + "> ::" + retstr);
  		  ((FTPThread)c).notifyError("FTPError <" + serveraddress.getHost() + "> ::" + retstr);
  		  throw new ConnectionException("FTPError <" + serveraddress.getHost() + "> ::" + retstr);
  		}		
  	}
  	else
  	if((retstr.equals("530")) || // Not logged in
  		 (retstr.equals("332")) || // account needed
  		 (retstr.equals("500")) || // Syntax error, command unrecognized
  		 (retstr.equals("501")) || // Syntax error in parameters or arguments
  		 (retstr.equals("421")))  {  // Service not available, closing control connection
  		System.out.println(retstr);
 		  ((FTPThread)c).notifyError("FTPError <" + serveraddress.getHost() + "> ::" + retstr);
 		  throw new ConnectionException("FTPError <" + serveraddress.getHost() + "> ::" + retstr);
  	}
  	
  	opencmd = "cwd " + _remotePath;
  	c.writeMsg(opencmd);
System.out.println(opencmd);

  	retstr = c.getOneMessage();
System.out.println(retstr);
		// various error condition if not 250  	
  	if(!retstr.startsWith("250"))  {
  		// if directory not found and want to create, make directory
  		if((retstr.startsWith("550")) && _createMode)  {
  			opencmd = "mkd " + _remotePath;
  			c.writeMsg(opencmd);
System.out.println(opencmd);
  			
  			retstr = c.getOneMessage();
System.out.println(retstr);
  			
  			// directory created. Change directory
  			if(retstr.startsWith("257"))  {
  				opencmd = "cwd " + _remotePath;
  				c.writeMsg(opencmd);
System.out.println(opencmd);
  				
  			}
  			else  {	
  				System.out.println(retstr);
		 		  ((FTPThread)c).notifyError("FTPError <" + serveraddress.getHost() + "> ::" + retstr);
		 		  throw new ConnectionException("FTPError <" + serveraddress.getHost() + "> ::" + retstr);
  			}
  		}
  		else  {		
  			System.out.println(retstr);
	 		  ((FTPThread)c).notifyError("FTPError <" + serveraddress.getHost() + "> ::" + retstr);
	 		  throw new ConnectionException("FTPError <" + serveraddress.getHost() + "> ::" + retstr);
  		}	
  	}	
  }	

	/**
	* Send disconnect message to the server. The ReceiverThread is initiated
	* by me.
	* @param c ReceiverThread, which will stop.
	* @param myid My name, may be passed to the server to disconnect gracefully
	*/
  public void processClientLogout(ReceiverThread c) {
  	try  {
  		c.writeMsg("QUIT");
  	}  catch (ConnectionException e)  {}	
  } 
  
  /**
  * No implementation for SMTP
  * @param obj Object which dependent upon the security implementation
  * @return true if the agent is valid else, retur false
  */
  public boolean isValidAgent(Object o)  {
  	return true;
  }	
  
  /**
    * Set create directory mode. 
    * @param isCreateDir True if want to create directory
    */
  public void setMode(boolean isCreateDir)  {
  	_createMode = isCreateDir;
  }	
  
}		
			
	
				