package ProtocolLayer;

import Abstract.*;

import java.util.*;

/**
  * Abstraction for sending KQML message through email
  */
  
/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 
  
public class SMTPCon  {
	

	/**
	  * Email server address
	  */
	Address _serverAddress;
	
	/**
	  * My email address
	 */ 
	String _myEmailAddress;
	
	/**
	  * Receivers' email address
	  */
	String[] _receiverEmailAddresses;
	
	/**
	  * Email content(KQMLmessage)
    */	  	
	String _data;
	
	/**
	  * Error message buffer
	  */
	String _errorMsg = null;
	
	/**
	  * Default constructor
	  */
	public SMTPCon()  {
		_serverAddress = null;
		_myEmailAddress = "";
		_receiverEmailAddresses = null;
		_data = "";
	}	
		
	/**
	  * Constructor using server address, my address, receivers' address and data
	  * @param serverAddress Email server address
	  * @param myAddr My email address
	  * @param recAddrs Receivers' email address
	  * @param data KQML message to be sent
	  */ 	
	public SMTPCon(Address serverAddress,
							   String myAddr,
							   String[] recAddrs,
							   String data)  {
		_serverAddress = serverAddress;
		_myEmailAddress =  myAddr;
		_receiverEmailAddresses = recAddrs;
		_data = data;
	}

	/**
	  * Set email server address
	  * @param addr Email server address
	  */
	public void setServerAddress(Address addr)  {
		_serverAddress = addr;
	}
	
	/**
	  * Get email server address
	  * @return Email server address
	  */
	public Address getServerAddress()  {
		return _serverAddress;
	}
	
	/**
	  * Set my email address(e.g. jhc@cdr.stanford.edu
	  * @param addr My email address
	  */
	public void setMyAddress(String addr)  {
		_myEmailAddress = addr;
	}
	
	/**
	  * Get my email address
	  * @return my email address(e.g. jhc@cdr.stanford.edu)
  	*/
	public String getMyAddress() {
		return _myEmailAddress;
	}

  /**
    * Set receivers' email address
    * @param s Receivers' email address
    */	
	public void setReceiverAddresses(String[] s)  {
		_receiverEmailAddresses = s;
	}
	
	/**
	  * Get receivers' email address
	  * @return Receivers' email address
	  */
	public String[] getReceiverAddresses()  {
		return _receiverEmailAddresses;
	}
	
	/**
	  * Set data to be sent
	  * @param data KQML message to be sent
	  */
	public void setData(String data)  {
		_data = data;
	}
	
	/**
	  * Get data to be sent
	  * @return KQML message to be sent
	  */
	public String getData()  {
		return _data;
	}
	
	/**
	  * Make a string list for this connection data.<BR> For instance :<P>
	  * String[0] = SMTP,cdr.stanford.edu,25,SMTP<BR>
	  * String[1] = jhc@cdr.stanford.edu<BR>
	  * String[2] = (mary@abc.com tom@nbc.com)<BR>
	  * String[3] = (tell :sender jhcAgent :receiver maryAgent :content (test this))<BR>
	  * String[4] = "550 error message"<P>
	  * @return String list
	  */
	public String[] toStringList()  {
		String[] list = new String[5];
		list[0] = _serverAddress.toString();
		list[1] = _myEmailAddress;
		list[3] = _data;
		list[4] = _errorMsg;
		if(_receiverEmailAddresses != null)  {
			list[2] = "(";
			for(int i=0; i < _receiverEmailAddresses.length; i++)  {
				list[2] = list[2] + _receiverEmailAddresses[i] + " ";
			}
			list[2] = list[2] + ")";
		}
		else  {
			list[2] = null;
		}
		return list;				
	}
	
	/**
	  * Set error message if error occurs
	  * @param err Error message from SMTP
	  */
	public void setErrorMessage(String err)  {
		_errorMsg = err;
	}
	
	/**
	  * Get error string
	  * @return Error message from SMTP transaction
	  */
	public String getErrorMessage()  {
		return _errorMsg;
	}		
	
}	