/**
 * @(#)ArrowButtonFace.java	2.2 11/25/96 Rodney Ryan (rer@interport.net)
 *
 * Copyright (c) 1996 Rodney Ryan, All Rights Reserved.
 *
 */

package RouterLayer.AgentClient.Example.RCApplet.extawt;

import java.awt.*;

/**
 * Provides an ButtonFace interface representing the 4 directional arrows.
 *
 * @author 	Rodney Ryan (rer@interport.net)
 * @version 	2.2 11/25/96
 *
 */

public class ArrowButtonFace extends Object implements ButtonFace
{
    /**
    * Arrow points north.
    */
    public static final int NORTH = 0;

    /**
    * Arrow points south.
    */
    public static final int SOUTH = 1;

    /**
    * Arrow points east.
    */
    public static final int EAST  = 2;

    /**
    * Arrow points west.
    */
    public static final int WEST  = 3;

    /**
    * Variable indicating the direction the arrow points.
    */
    private int arrowDirection;

    /**
    * Variable indicating the size.
    */
    private Dimension size = new Dimension(12, 12);

    /**
    * Constructs a new instance of ArrowButtonFace.
    * @param arrowDirection the direction of the arrow
    */
    public ArrowButtonFace(int arrowDirection)
    {
        super();
        this.arrowDirection = arrowDirection;
    }

    /**
    * Paints the ArrowButtonFace.
    * @param g the Graphics
    * @param disabled the button's disabled status
    * @param pressed the button's pressed status
    */
    public void paint(Graphics g,
                      boolean  disabled,
                      boolean  pressed)
    {
        drawArrow(g, disabled, pressed);
    }

    /**
    * Draws the arrow.
    * @param g the Graphics
    * @param disabled the button's disabled status
    * @param pressed the button's pressed status
    */
    public void drawArrow(Graphics g,
                          boolean  disabled,
                          boolean  pressed)
    {
        switch (arrowDirection) {
            case ArrowButtonFace.NORTH:
                drawArrowNorth(g, disabled, pressed);
                break;
            case ArrowButtonFace.SOUTH:
                drawArrowSouth(g, disabled, pressed);
                break;
            case ArrowButtonFace.EAST:
                drawArrowEast(g, disabled, pressed);
                break;
            case ArrowButtonFace.WEST:
                drawArrowWest(g, disabled, pressed);
                break;
        }
    }

    /**
    * Draws the arrow pointed north.
    * @param g the Graphics
    * @param disabled the button's disabled status
    * @param pressed the button's pressed status
    */
    public void drawArrowNorth(Graphics g,
                               boolean  disabled,
                               boolean  pressed)
    {
        int x = (size.width / 2);
        int y = (size.height / 2);
        if (pressed) {
            x++;
            y++;
        }
        g.setColor(Color.black);
        for (int i = 0; i < 4; i++) g.drawLine(x - i, y + i, x + i, y + i);
    }

    /**
    * Draws the arrow pointed south.
    * @param g the Graphics
    * @param disabled the button's disabled status
    * @param pressed the button's pressed status
    */
    public void drawArrowSouth(Graphics g,
                               boolean  disabled,
                               boolean  pressed)
    {
        int x = (size.width / 2);
        int y = (size.height / 2);
        if (pressed) {
            x++;
            y++;
        }
        g.setColor(Color.black);
        for (int i = 0; i < 4; i++) g.drawLine(x + i - 3, y + i, x + 3 - i, y + i);
    }

    /**
    * Draws the arrow pointed east.
    * @param g the Graphics
    * @param disabled the button's disabled status
    * @param pressed the button's pressed status
    */
    public void drawArrowEast(Graphics g,
                              boolean  disabled,
                              boolean  pressed)
    {
        int x = (size.width / 2);
        int y = (size.height / 2) + 1;
        if (pressed) {
            x++;
            y++;
        }
        if (disabled) g.setColor(Color.gray);
            else g.setColor(Color.black);
        for (int i = 0; i < 4; i++) g.drawLine(x + i, y - 3 + i, x + i, y + 3 - i);
        if (disabled) {
            g.setColor(Color.white);
            g.drawLine(x + 4, y + 1, x + 1, y + 4);
            g.drawLine(x + 3, y + 1, x + 1, y + 3);
        }
    }

    /**
    * Draws the arrow pointed west.
    * @param g the Graphics
    * @param disabled the button's disabled status
    * @param pressed the button's pressed status
    */
    public void drawArrowWest(Graphics g,
                              boolean  disabled,
                              boolean  pressed)
    {
        int x = (size.width / 2) - 1;
        int y = (size.height / 2) + 1;
        if (pressed) {
            x++;
            y++;
        }
        if (disabled) g.setColor(Color.gray);
            else g.setColor(Color.black);
        for (int i = 0; i < 4; i++) g.drawLine(x + i, y - i, x + i, y + i);
        if (disabled) {
            g.setColor(Color.white);
            g.drawLine(x + 4, y - 3, x + 4, y + 4);
        }

    }

    /**
    * Returns the size of the ArrowButtonFace.
    */
    public Dimension size()
    {
        return size;
    }
}