package RouterLayer.AgentClient;


import KQMLLayer.*;

import java.util.*;
import java.lang.*;
import java.text.*;

/**
* KQMLmail object. Wrap the KQMLmessage and keep the message
* number. KQMLmail will be used if you use GUI to show simple
* message contents - readbit, received time, sender, performative
* KQMLmail syntax:<BR>
* <UL>
* <LI>readbit [space] timestamp [space] KQMLString<BR>
* readbit := N || F || R, N for not read, F for reserved, R for received reserved message<BR>
* timestamp := String, String is 12 byte character number(e.g. 887463524231, which will
* be used to construct Date or Calandar object<BR>
* KQMLString := KQML syntax string. This is actual KQML message<BR>
* </UL>
*/
/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class KQMLmail  {

		/**
		* Message number, starting from the beginning
		*/
		int _messageNumber;
		
		/**
		* Time stamp
		*/
		Long _time;
		
		/**
		* Readable bit
		*/
		char _readbit;
		
		/**
		* KQMLmessage
		*/
    KQMLmessage _kqmlmessage;

		/**
		* Constructor
		* @param msg KQML message to be sent or received
		* @param messageno Message index
		*/
    public KQMLmail(String msg,int messageno)
        throws KQMLLayer.ParseException   {
			_readbit = msg.charAt(0);
			String stamp = msg.substring(2,14);	
			String content = msg.substring(15,msg.length());
			_kqmlmessage = new KQMLmessage(content);
			
			_time = new Long(stamp);
			
			_messageNumber = messageno;
		}		
			
		/**
		* Get message index
		* @return Message index
		*/		
		public int getMessageNumber()   {
			return _messageNumber;
		}
		
		/**
		* Set message index
		* @param no Message index to be set
		*/
		public void setMessageNumber(int no)  {
			_messageNumber = no;
		}	
		
		/**
		* Get KQMLmessage
		* @return KQMLmessage, wrapped by KQMLmail
		*/
    public KQMLmessage getKQMLmessage()   {
      return _kqmlmessage;
    }

		/**
		* Get read bit character
		* @return Readbit
		*/
		public char getReadBit()  {
			return _readbit;
		}
		
		/**
		* Set read bit character
		* @param c Read bit to be set
		*/
		public void setReadBit(char c)  {
			_readbit = c;
		}
						
		/**
		* Get time stamp
		* @return Time stamp
		*/			
		public Long getTime()   {
			return _time;
		}
			
		/**
		* Get human readable time string from the time stamp				
		* @return Readable time string
		*/
    protected String getTimeString()  {
  		SimpleDateFormat df = new SimpleDateFormat("MMM.dd.yyyyy hh:mm aaa");
  		return (df.format(new Date(_time.longValue())));
    }	
			
    /**
    * Get KQMLmail stamp
    * @return KQMLmail stamp, which is readable to human
    */
    public String getString()   {
        String perf = _kqmlmessage.getValue("performative");
				String sender = _kqmlmessage.getValue("sender");
        return (_readbit + "   " + _messageNumber + "   " +  
        				sender + "   " + perf + "   " + getTimeString());
    }
    
    static public void main(String args[])  {
    	String test = "N ";
    	long t = System.currentTimeMillis();
    	test = test + Long.toString(t) + "\0(tell :sender me)";
    	try  {
    		KQMLmail mail = new KQMLmail(test,5);
    		System.out.println(mail.getString());
    	} catch (Exception e)  {	
    		System.out.println(e.toString());
    	}	
    		
	 }	 
}