package Setup;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.applet.*;
import java.io.*;

import Abstract.*;

import RouterLayer.AgentClient.Example.RCApplet.extawt.*;

import ProtocolLayer.*;


/**
* Setup JATLite to modify address and html files
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*
*/ 

public class SetupFrame extends Frame implements ActionListener {
	
	//GUI variables
	private Font normFont = new Font("Dialog",Font.PLAIN,10),
				 boldFont = new Font("Dialog",Font.BOLD,10),
				 ultraFont = new Font("Times", Font.PLAIN,18),
				 ultraBoldFont = new Font("Times", Font.BOLD,18),
				 bigFont = new Font("Times",Font.PLAIN,14),
				 bigBoldFont = new Font("Times",Font.BOLD,14);
				 
	private Color	bkgColor = new Color(210,210,210),
					scrlBarColor = new Color(225,225,225);
					
	private Panel 	wholePanel, upperPanel, lowerPanel, centerPanel,routerPanel, ftpPanel /*otherPanel*/;                                                  
	TabFolder tf;                                                                  
	SetupRouterPanel _routerp;
	SetupFTPPanel _ftpp;
	
	TextField _filePath;
	String _modTxtFile;
	
	// Router address
	Address _routerAddress;
	
	// Registrar address
	Address _registrarAddress;

	int _durationTime;
	
	FTPCon _ftpCon;
	String _email;
		
	public SetupFrame()  {
		super("JATLite Example setup");

		setLayout(new BorderLayout());
		setFont(normFont);
		setSize(340, 400);
		add("Center", AddressPanels());
  }

  private Panel newPackerPanel()	{
		Panel p = new Panel();
		p.setLayout(new PackerLayout());
		return p;
	}
	
	private Panel AddressPanels() 	{	
	
		Button browse = new Button("Browse");
		browse.addActionListener(this);
		
		Button start = new Button("Start");
		start.addActionListener(this);
		
		Button close = new Button("Quit");
		close.addActionListener(this);
		
	
		_routerp = new SetupRouterPanel(this);
		_ftpp = new SetupFTPPanel(this);
	
		wholePanel = newPackerPanel();
		wholePanel.setBackground(bkgColor);
		
		upperPanel = newPackerPanel();
		Label filepath = new Label("JATLite Directory",Label.LEFT);
		_filePath = new TextField("",20);
		upperPanel.add("label;side=right;expand=true;padx=3;fill=both",browse);
		upperPanel.add("label;side=right;expand=true;padx=3;fill=both",_filePath);
		upperPanel.add("label;side=right;expand=true;padx=3",filepath);


		centerPanel = newPackerPanel();		
		routerPanel = newPackerPanel();
		routerPanel.add("panel;expand=true;fill=both", _routerp);
	
		ftpPanel = newPackerPanel();
		ftpPanel.add("panel;expand=true;fill=both", _ftpp);
		
		tf = new TabFolder("Router", routerPanel);
	  tf.addTab("FTP", ftpPanel);
    centerPanel.add("TabFolder;side=top;pady=5", tf);
		
		lowerPanel = newPackerPanel();
		lowerPanel.add("label;side=right;expand=true;padx=10;fill=both", close);
		lowerPanel.add("label;side=right;expand=true;padx=10;fill=both", start);
		
		wholePanel.add("panel;side=top;padx=5;pady=10", upperPanel);
		wholePanel.add("panel;side=top;padx=5;pady=5", centerPanel);
		wholePanel.add("panel;side=top;expand=false;padx=5;pady=5", lowerPanel);
				
		return wholePanel;
	}


  /**
  * ActionPerformed implementation
  */
	public void actionPerformed(ActionEvent event) {
      String source = event.getActionCommand();

      if(source.equals("Browse"))  {
		   	browseAction();
      }
      else
      if(source.equals("Start"))  {
		   	startAction();
      }
      else
      if(source.equals("Quit"))  {
		   	quitAction();
      }
  }  

	protected void browseAction()  {
		FileDialog fd = new FileDialog(this,"Open Address File");
		fd.setFile("ModFiles.txt");
		fd.show();
		String dir = fd.getDirectory();
		_modTxtFile = fd.getFile();
		_filePath.setText(dir);
	}	
		
			
	protected String getAddressString(Address addr)  {
		String retstr = addr.getID();
		retstr = retstr + "," + addr.getHost() + "," + Integer.toString(addr.getPort()) + ",";
		retstr = retstr + addr.getType() + "," + addr.getDescription();
		return retstr;
	}		
	
	protected void startAction()  {
		_routerAddress = _routerp.getRouterAddress();
		_registrarAddress = _routerp.getRegistrarAddress();
		_durationTime = _routerp.getDurationTime();
		
		_ftpCon = _ftpp.getFTPCon();
		_email = _ftpp.getEmail();
		
		if((_routerAddress != null) && (_registrarAddress != null)) {
			try  {
				updateFiles();
			} catch (Exception e)  {
  		 	System.out.println(e.toString());
			}
		}
		else {
			System.out.println("My address and Router address should be specified");
		}		
		
	}		
	
	protected void updateFiles()  {
		try  {
			String filePath = _filePath.getText();
			String line;
			BufferedReader in = new BufferedReader(new FileReader(filePath  + _modTxtFile));
		
			while((line = in.readLine()) != null)  {
		
				if(line.equals(""))  {
					break;
				}
					
				BufferedWriter tmp = new BufferedWriter(new FileWriter(filePath  +  "setup.tmp"));
				
				BufferedReader reader = new BufferedReader(new FileReader(filePath   + line + ".bak"));
				String fline;
			
				while((fline = reader.readLine()) != null)  {

					// replace JATLite directory
					String tmpl = replaceLine(fline,"%JATLiteDirectory%",filePath);
					tmpl = replaceLine(tmpl,"%RouterName%",_routerAddress.getID());
					tmpl = replaceLine(tmpl,"%RouterHost%",_routerAddress.getHost());
					tmpl = replaceLine(tmpl,"%RouterPort%",Integer.toString(_routerAddress.getPort()));
					tmpl = replaceLine(tmpl,"%RegistrarName%",_registrarAddress.getID());
					tmpl = replaceLine(tmpl,"%RegistrarPort%",Integer.toString(_registrarAddress.getPort()));
					tmpl = replaceLine(tmpl,"%DurationTime%",Integer.toString(_durationTime));
					tmpl = replaceLine(tmpl,"%FTPConRemotePath%",_ftpCon.getRemotePath());
					tmpl = replaceLine(tmpl,"%FTPConHost%",_ftpCon.getHost());
					tmpl = replaceLine(tmpl,"%FTPConPort%",Integer.toString(_ftpCon.getPort()));
					tmpl = replaceLine(tmpl,"%FTPConUserName%",_ftpCon.getUserName());
					tmpl = replaceLine(tmpl,"%FTPConPassword%",_ftpCon.getPassword());
					tmpl = replaceLine(tmpl,"%FTPConLocalPath%",_ftpCon.getLocalPath());
					tmpl = replaceLine(tmpl,"%Email%",_email);
				
					tmpl += "\n";
				
					tmp.write(tmpl);							
							
				}	
					
				reader.close();		
			
				BufferedWriter out = new BufferedWriter(new FileWriter(filePath   + line));
				tmp.close();
				
				BufferedReader rtmp = new BufferedReader(new FileReader(filePath  + "setup.tmp"));
			
				while((fline = rtmp.readLine()) != null)  {
					out.write(fline + "\n");
				}
				out.close();
				rtmp.close();	
			}
		
			in.close();		
			System.out.println("Setup Success");
			System.exit(0);
			
		} catch (Exception e)  {
			System.out.println(e.toString());
			System.exit(0);
		}	
	}	

	protected String replaceLine(String original, String replaced,String newstr)  {
		String tmpline = original;
		int index = original.indexOf(replaced);
		if(index != -1)   {
			tmpline = original.substring(0,index);
			tmpline = tmpline + newstr;
			tmpline = tmpline + original.substring(index + replaced.length());
		}
		
		return tmpline;
	}
		
	protected void quitAction()  {
		dispose();
		System.exit(0);
	}

	private int getInt(String str)	{
		return (Integer.valueOf(str)).intValue();
	}
	
	public static void main(String args[])   {
		SetupFrame f = new SetupFrame();
		f.setVisible(true);
	}	
}		